@@ -1,6 +1,7 @@
 import functools
+import json as complexjson
 import numbers
-from typing import Any, Dict, List, Union
+from typing import Any, Dict, Generator, List, Union
 
 from cognite.client._api_client import APIClient
 
@@ -11,6 +12,7 @@ from cognite.experimental.data_classes.geospatial import (
     FeatureList,
     FeatureType,
     FeatureTypeList,
+    FeatureTypeUpdate,
 )
 
 
@@ -50,7 +52,7 @@ class ExperimentalGeospatialAPI(APIClient):
         <https://pr-1323.specs.preview.cogniteapp.com/v1.json.html#operation/createFeatureTypes>
 
         Args:
-            FeatureType (Union[FeatureType, List[FeatureType]]): feature type definition or list of feature type definitions to create.
+            feature_type (Union[FeatureType, List[FeatureType]]): feature type definition or list of feature type definitions to create.
 
         Returns:
             Union[FeatureType, FeatureTypeList]: Created feature type definition(s)
@@ -71,12 +73,13 @@ class ExperimentalGeospatialAPI(APIClient):
         return self._create_multiple(items=feature_type, cls=FeatureTypeList, resource_path="/spatial/featuretypes")
 
     @_with_cognite_domain
-    def delete_feature_types(self, external_id: Union[str, List[str]] = None) -> None:
+    def delete_feature_types(self, external_id: Union[str, List[str]], force: bool = False) -> None:
         """`Delete one or more feature type`
         <https://pr-1323.specs.preview.cogniteapp.com/v1.json.html#operation/deleteFeatureTypes>
 
         Args:
             external_id (Union[str, List[str]]): External ID or list of external ids
+            force (bool): if `true` the features will also be dropped
 
         Returns:
             None
@@ -89,7 +92,10 @@ class ExperimentalGeospatialAPI(APIClient):
                 >>> c = CogniteClient()
                 >>> c.geospatial.delete_feature_types(external_id=["wells", "pipelines"])
         """
-        return self._delete_multiple(external_ids=external_id, wrap_ids=True, resource_path="/spatial/featuretypes")
+        params = {"force": "true"} if force else None
+        return self._delete_multiple(
+            external_ids=external_id, wrap_ids=True, resource_path="/spatial/featuretypes", params=params
+        )
 
     @_with_cognite_domain
     def list_feature_types(self) -> FeatureTypeList:
@@ -133,6 +139,36 @@ class ExperimentalGeospatialAPI(APIClient):
             wrap_ids=True, external_ids=external_id, cls=FeatureTypeList, resource_path="/spatial/featuretypes"
         )
 
+    @_with_cognite_domain
+    def update_feature_types(self, update: Union[FeatureTypeUpdate, List[FeatureTypeUpdate]] = None) -> FeatureTypeList:
+        """`Patch feature types`
+        <https://pr-1323.specs.preview.cogniteapp.com/v1.json.html#operation/updateFeatureType>
+
+        Args:
+            update (Union[FeatureTypeUpdate, List[FeatureTypePatch]]): the update to apply
+
+        Returns:
+            FeatureTypeList: The updated feature types.
+
+        Examples:
+
+            Add one attribute to a feature type:
+
+                >>> from cognite.experimental import CogniteClient
+                >>> c = CogniteClient()
+                >>> res = c.geospatial.update_feature_types(update=FeatureTypeUpdate(external_id="wells", add=AttributeAndSearchSpec(attributes={"altitude": {"type": "DOUBLE"}})))
+        """
+        if isinstance(update, FeatureTypeUpdate):
+            update = [update]
+
+        mapper = lambda it: {
+            "attributes": None if not hasattr(it, "attributes") else it.attributes,
+            "searchSpec": None if not hasattr(it, "search_spec") else it.search_spec,
+        }
+        json = {"items": [{"externalId": it.external_id, "add": mapper(it.add)} for it in update]}
+        res = self._post(url_path=f"/spatial/featuretypes/update", json=json)
+        return FeatureTypeList._load(res.json()["items"], cognite_client=self._cognite_client)
+
     @_with_cognite_domain
     def create_features(
         self, feature_type: FeatureType, feature: Union[Feature, List[Feature]]
@@ -239,7 +275,9 @@ class ExperimentalGeospatialAPI(APIClient):
         return self._create_multiple(feature, resource_path=resource_path, cls=FeatureList)
 
     @_with_cognite_domain
-    def search_features(self, feature_type: FeatureType, filter: Dict[str, Any], limit: int = 100) -> FeatureList:
+    def search_features(
+        self, feature_type: FeatureType, filter: Dict[str, Any], attributes: Dict[str, Any] = None, limit: int = 100
+    ) -> FeatureList:
         """`Search for features`
         <https://pr-1323.specs.preview.cogniteapp.com/v1.json.html#operation/searchFeatures>
 
@@ -247,6 +285,7 @@ class ExperimentalGeospatialAPI(APIClient):
             feature_type: the feature type to search for
             filter (Dict[str, Any]): the search filter
             limit (int): maximum number of results
+            attributes (Dict[str, Any]): the output attribute selection
 
         Returns:
             FeatureList: the filtered features
@@ -262,13 +301,66 @@ class ExperimentalGeospatialAPI(APIClient):
                 >>> res = c.geospatial.search_features(my_feature_type, filter={"range": {"attribute": "temperature", "gt": 12.0}})
                 >>> for f in res:
                 ...     # do something with the features
+
+            Search for features and select output attributes:
+
+                >>> res = c.geospatial.search_features(my_feature_type, filter={}, attributes={"temperature": {}, "pressure": {}})
+
         """
         resource_path = self._feature_resource_path(feature_type) + "/search"
         cls = FeatureList
         resource_path = resource_path
-        res = self._post(url_path=resource_path, json={"filter": filter, "limit": limit},)
+        res = self._post(
+            url_path=resource_path, json={"filter": filter, "limit": limit, "output": {"attributes": attributes}}
+        )
         return cls._load(res.json()["items"], cognite_client=self._cognite_client)
 
+    def stream_features(
+        self, feature_type: FeatureType, filter: Dict[str, Any], attributes: Dict[str, Any] = None
+    ) -> Generator[Feature, None, None]:
+        """`Stream features`
+        <https://pr-1323.specs.preview.cogniteapp.com/v1.json.html#operation/streamFeatures>
+
+        Args:
+            feature_type: the feature type to search for
+            filter (Dict[str, Any]): the search filter
+            attributes (Dict[str, Any]): the output attribute selection
+
+        Returns:
+            Generator[Feature]: a generator for the filtered features
+
+        Examples:
+
+            Stream features:
+
+                >>> from cognite.experimental import CogniteClient
+                >>> c = CogniteClient()
+                >>> my_feature_type = c.geospatial.retrieve_feature_types(external_id="my_feature_type")
+                >>> my_feature = c.geospatial.create_features(my_feature_type, Feature(external_id="my_feature", temperature=12.4))
+                >>> features = c.geospatial.stream_features(my_feature_type, filter={"range": {"attribute": "temperature", "gt": 12.0}})
+                >>> for f in features:
+                ...     # do something with the features
+
+            Stream features and select output attributes:
+
+                >>> features = c.geospatial.stream_features(my_feature_type, filter={}, attributes={"temperature": {}, "pressure": {}})
+                >>> for f in features:
+                ...     # do something with the features
+
+        """
+        resource_path = self._feature_resource_path(feature_type) + "/search-streaming"
+        resource_path = resource_path
+        json = {"filter": filter, "output": {"attributes": attributes, "jsonStreamFormat": "NEW_LINE_DELIMITED"}}
+
+        self._config.headers.pop(self.X_COGNITE_DOMAIN, None)
+        if self._cognite_domain is not None:
+            self._config.headers.update({self.X_COGNITE_DOMAIN: feature_type._cognite_domain})
+        res = self._do_request("POST", url_path=resource_path, json=json, timeout=self._config.timeout, stream=True)
+        self._config.headers.pop(self.X_COGNITE_DOMAIN, None)
+
+        for line in res.iter_lines():
+            yield Feature._load(complexjson.loads(line))
+
     def get_coordinate_reference_systems(self, srids: Union[int, List[int]] = None) -> CoordinateReferenceSystemList:
         """`Get Coordinate Reference Systems`
         <https://pr-1323.specs.preview.cogniteapp.com/v1.json.html#operation/getCRS>
